==> modonomicon-1.21.5-fabric-1.117.2-sources.jar: com/klikli_dev/modonomicon/api/datagen/CategoryEntryMap.java <==
/*
 * SPDX-FileCopyrightText: 2022 klikli-dev
 *
 * SPDX-License-Identifier: MIT
 */

package com.klikli_dev.modonomicon.api.datagen;

import com.mojang.logging.LogUtils;
import org.slf4j.Logger;

import java.util.Arrays;
import net.minecraft.class_241;

/**
 * A helper class to visualize entry positions within a category.
 * Allows to map characters to positions in a 2D grid which will be used as coordinates for entries within the category for rendering.
 */
public class CategoryEntryMap {

    public static final Logger LOGGER = LogUtils.getLogger();

    protected String[] map;

    protected class_241 offset;
    protected class_241 additionalOffset;

    public CategoryEntryMap() {
        this.additionalOffset = class_241.field_1340;
    }

    public void setMap(String... map) {
        //we remove whitespaces, including tabs
        this.map = Arrays.stream(map).map(s -> s.replaceAll("\\s", "")).toArray(String[]::new);

        this.offset = new class_241(-(int) (map[0].length() / 2.0f), -(int) (map.length / 2.0f));
    }

    /**
     * Overwrites the automatically calculated offset for entry coordinates.
     * In most cases you will instead want to use setAdditionalOffset().
     * Call in CategoryProvider#additionalSetup (or at any rate, after setMap();
     */
    public void setOffset(class_241 offset) {
        this.offset = offset;
    }

    /**
     * Overwrites the automatically calculated offset for entry coordinates.
     * In most cases you will instead want to use setAdditionalOffset().
     * Call in CategoryProvider#additionalSetup (or at any rate, after setMap();
     */
    public void setOffset(int x, int y) {
        this.offset = new class_241(x, y);
    }

    /**
     * Adds an additional offset to the calculated offset for entry coordinates.
     */
    public void setAdditionalOffset(class_241 offset) {
        this.additionalOffset = offset;
    }

    /**
     * Adds an additional offset to the calculated offset for entry coordinates.
     */
    public void setAdditionalOffset(int x, int y) {
        this.additionalOffset = new class_241(x, y);
    }

    /**
     * Matches a single character in the map as a symbol to represent an entry.
     */
    public class_241 get(Character symbol) {
        int y = 0;
        for (var line : this.map) {
            int x = 0;

            //get x coordinate in line. Each char is one position, but all characters within ( and ) count only as one
            boolean inBracket = false;
            for (var c : line.toCharArray()) {
                //do not match a char symbol within brackets, because everything in there is part of a larger string symbol
                if (c == symbol && !inBracket) {
                    return new class_241(x, y).method_35586(this.offset).method_35586(this.additionalOffset);
                } else if (c == '(') {
                    inBracket = true;
                } else if (c == ')') {
                    inBracket = false;
                    x++;
                } else if (!inBracket) {
                    x++;
                }
            }
            y++;
        }
        LOGGER.warn("Symbol '{}' not found in map", symbol);
        return class_241.field_1340;
    }

    /**
     * Matches a string in the map as a symbol wrapped in ( and ) to represent an entry.
     * The string must be surrounded by ( and ) in the map and may not contain any whitespace characters.
     * You can hand over the symbol with or without brackets to this method.
     */
    public class_241 get(String symbol) {

        symbol = symbol.replace("(", "");
        symbol = symbol.replace(")", "");

        int y = 0;
        for (var line : this.map) {
            int x = 0;
            //get x coordinate in line. Each char is one position, but all characters within ( and ) count only as one

            boolean inBracket = false;
            String currentSymbol = "";
            for (var c : line.toCharArray()) {
                if (c == '(') {
                    inBracket = true;
                } else if (c == ')') {
                    inBracket = false;
                    if (currentSymbol.equals(symbol)) {
                        return new class_241(x, y).method_35586(this.offset).method_35586(this.additionalOffset);
                    }
                    x++;
                    currentSymbol = "";
                } else if (inBracket) {
                    currentSymbol += c;
                } else {
                    x++;
                }
            }
            y++;
        }
        LOGGER.warn("Symbol '({})' not found in map", symbol);
        return class_241.field_1340;
    }
}

==> modonomicon-1.21.8-fabric-1.117.1-sources.jar: com/klikli_dev/modonomicon/api/datagen/CategoryEntryMap.java <==
/*
 * SPDX-FileCopyrightText: 2022 klikli-dev
 *
 * SPDX-License-Identifier: MIT
 */

package com.klikli_dev.modonomicon.api.datagen;

import com.mojang.logging.LogUtils;
import org.slf4j.Logger;

import java.util.Arrays;
import net.minecraft.class_241;

/**
 * A helper class to visualize entry positions within a category.
 * Allows to map characters to positions in a 2D grid which will be used as coordinates for entries within the category for rendering.
 */
public class CategoryEntryMap {

    public static final Logger LOGGER = LogUtils.getLogger();

    protected String[] map;

    protected class_241 offset;
    protected class_241 additionalOffset;

    public CategoryEntryMap() {
        this.additionalOffset = class_241.field_1340;
    }

    public void setMap(String... map) {
        //we remove whitespaces, including tabs
        this.map = Arrays.stream(map).map(s -> s.replaceAll("\\s", "")).toArray(String[]::new);

        this.offset = new class_241(-(int) (this.map[0].length() / 2.0f), -(int) (this.map.length / 2.0f));
    }

    /**
     * Overwrites the automatically calculated offset for entry coordinates.
     * In most cases you will instead want to use setAdditionalOffset().
     * Call in CategoryProvider#additionalSetup (or at any rate, after setMap();
     */
    public void setOffset(class_241 offset) {
        this.offset = offset;
    }

    /**
     * Overwrites the automatically calculated offset for entry coordinates.
     * In most cases you will instead want to use setAdditionalOffset().
     * Call in CategoryProvider#additionalSetup (or at any rate, after setMap();
     */
    public void setOffset(int x, int y) {
        this.offset = new class_241(x, y);
    }

    /**
     * Adds an additional offset to the calculated offset for entry coordinates.
     */
    public void setAdditionalOffset(class_241 offset) {
        this.additionalOffset = offset;
    }

    /**
     * Adds an additional offset to the calculated offset for entry coordinates.
     */
    public void setAdditionalOffset(int x, int y) {
        this.additionalOffset = new class_241(x, y);
    }

    /**
     * Matches a single character in the map as a symbol to represent an entry.
     */
    public class_241 get(Character symbol) {
        int y = 0;
        for (var line : this.map) {
            int x = 0;

            //get x coordinate in line. Each char is one position, but all characters within ( and ) count only as one
            boolean inBracket = false;
            for (var c : line.toCharArray()) {
                //do not match a char symbol within brackets, because everything in there is part of a larger string symbol
                if (c == symbol && !inBracket) {
                    return new class_241(x, y).method_35586(this.offset).method_35586(this.additionalOffset);
                } else if (c == '(') {
                    inBracket = true;
                } else if (c == ')') {
                    inBracket = false;
                    x++;
                } else if (!inBracket) {
                    x++;
                }
            }
            y++;
        }
        LOGGER.warn("Symbol '{}' not found in map", symbol);
        return class_241.field_1340;
    }

    /**
     * Matches a string in the map as a symbol wrapped in ( and ) to represent an entry.
     * The string must be surrounded by ( and ) in the map and may not contain any whitespace characters.
     * You can hand over the symbol with or without brackets to this method.
     */
    public class_241 get(String symbol) {

        symbol = symbol.replace("(", "");
        symbol = symbol.replace(")", "");

        int y = 0;
        for (var line : this.map) {
            int x = 0;
            //get x coordinate in line. Each char is one position, but all characters within ( and ) count only as one

            boolean inBracket = false;
            String currentSymbol = "";
            for (var c : line.toCharArray()) {
                if (c == '(') {
                    inBracket = true;
                } else if (c == ')') {
                    inBracket = false;
                    if (currentSymbol.equals(symbol)) {
                        return new class_241(x, y).method_35586(this.offset).method_35586(this.additionalOffset);
                    }
                    x++;
                    currentSymbol = "";
                } else if (inBracket) {
                    currentSymbol += c;
                } else {
                    x++;
                }
            }
            y++;
        }
        LOGGER.warn("Symbol '({})' not found in map", symbol);
        return class_241.field_1340;
    }
}
